require 'rails_helper'
require 'thread'
require 'thwait'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')


RSpec.describe HousingRequestEntityUpdateJob do
  subject { HousingRequestEntityUpdateJob }
  let!(:siret1) { '12345678901234' }
  let!(:siret2) { '22345678901234' }
  let!(:entity) { create(:entity, siret: siret1) }
  let!(:app1) { create(:applicant, main_employer_siret: siret1)}
  let!(:hr_with_siret1) { create(:housing_request, main_applicant: app1)}
  let!(:app2) { create(:applicant, main_employer_siret: siret2)}
  let!(:hr_with_siret2) { create(:housing_request, main_applicant: app2)}
  let!(:app3) { create(:applicant)}
  let!(:coapp3) { create(:applicant, main_employer_siret: siret1)}
  let!(:hr_with_siret1_used_for_siret) { create(:housing_request, main_applicant: app3, applicant_used_for_siret: coapp3.id)}
  let!(:app4) { create(:applicant)}
  let!(:coapp4) { create(:applicant, main_employer_siret: siret2)}
  let!(:hr_with_siret2_used_for_siret) { create(:housing_request, main_applicant: app4, applicant_used_for_siret: coapp4.id)}
  let!(:app5) { create(:applicant, main_employer_siret: siret1[0..8] + "00000")}
  let!(:hr_with_siren1) { create(:housing_request, main_applicant: app5)}
  let!(:app6) { create(:applicant, main_employer_siret: siret2[0..8] + "00000")}
  let!(:hr_with_siren2) { create(:housing_request, main_applicant: app6)}
  let!(:hr_with_siren2_form) { create(:entity_form, housing_request_id: hr_with_siren2.id, level: 2)}

  before :each do 
    Hermes::HousingRequest.each do |hr|
      hr.set(consent_level: 2)
    end 
  end 

  it 'should create jobs' do
    expect do
      entity.update(siret: siret2)
    end.to change { enqueued_jobs.select{|job| job[:job] == subject }.length }.from(1).to(2)
  end

  it 'when creating entity' do 
    expect(hr_with_siret1.reload.entity_id).to eq(entity.id)
    expect(hr_with_siret1.consent_level).to eq(2)
    expect(hr_with_siren1.reload.entity_id).to eq(entity.id)
    expect(hr_with_siren1.consent_level).to eq(2)
    expect(hr_with_siret1_used_for_siret.reload.entity_id).to eq(entity.id)
    expect(hr_with_siret1_used_for_siret.consent_level).to eq(2)
    expect(hr_with_siret2.reload.entity_id).to eq(nil)
    expect(hr_with_siret2.consent_level).to eq(2)
    expect(hr_with_siren2.reload.entity_id).to eq(nil)
    expect(hr_with_siren2.consent_level).to eq(2)
    expect(hr_with_siret2_used_for_siret.reload.entity_id).to eq(nil)
    expect(hr_with_siret2_used_for_siret.consent_level).to eq(2)

    e = Hermes::Entity.create(siret: siret2, siren: siret2[0..8])
    execute_last_job(HousingRequestEntityUpdateJob)

    expect(hr_with_siret1.reload.entity_id).to eq(entity.id)
    expect(hr_with_siret1.consent_level).to eq(2)
    expect(hr_with_siren1.reload.entity_id).to eq(entity.id)
    expect(hr_with_siren1.consent_level).to eq(2)
    expect(hr_with_siret1_used_for_siret.reload.entity_id).to eq(entity.id)
    expect(hr_with_siret1_used_for_siret.consent_level).to eq(2)
    expect(hr_with_siret2.reload.entity_id).to eq(e.id)
    expect(hr_with_siret2.consent_level).to eq(2)
    expect(hr_with_siren2.reload.entity_id).to eq(e.id)
    expect(hr_with_siren2.consent_level).to eq(2)
    expect(hr_with_siret2_used_for_siret.reload.entity_id).to eq(e.id)
    expect(hr_with_siret2_used_for_siret.consent_level).to eq(2)
  end 

  it 'should changes the values' do
    expect(hr_with_siret1.reload.entity_id).to eq(entity.id)
    expect(hr_with_siret1.consent_level).to eq(2)
    expect(hr_with_siren1.reload.entity_id).to eq(entity.id)
    expect(hr_with_siren1.consent_level).to eq(2)
    expect(hr_with_siret1_used_for_siret.reload.entity_id).to eq(entity.id)
    expect(hr_with_siret1_used_for_siret.consent_level).to eq(2)
    expect(hr_with_siret2.reload.entity_id).to eq(nil)
    expect(hr_with_siret2.consent_level).to eq(2)
    expect(hr_with_siren2.reload.entity_id).to eq(nil)
    expect(hr_with_siren2.consent_level).to eq(2)
    expect(hr_with_siret2_used_for_siret.reload.entity_id).to eq(nil)
    expect(hr_with_siret2_used_for_siret.consent_level).to eq(2)

    entity.update(siret: siret2, siren: siret2[0..8])
    execute_last_job(HousingRequestEntityUpdateJob)

    expect(hr_with_siret1.reload.entity_id).to eq(nil)
    expect(hr_with_siret1.consent_level).to eq(2)
    expect(hr_with_siren1.reload.entity_id).to eq(nil)
    expect(hr_with_siren1.consent_level).to eq(2)
    expect(hr_with_siret1_used_for_siret.reload.entity_id).to eq(nil)
    expect(hr_with_siret1_used_for_siret.consent_level).to eq(2)
    expect(hr_with_siret2.reload.entity_id).to eq(entity.id)
    expect(hr_with_siret2.consent_level).to eq(2)
    expect(hr_with_siren2.reload.entity_id).to eq(entity.id)
    expect(hr_with_siren2.consent_level).to eq(2)
    expect(hr_with_siret2_used_for_siret.reload.entity_id).to eq(entity.id)
    expect(hr_with_siret2_used_for_siret.consent_level).to eq(2)
  end 

  it 'create log error' do 
    Hermes::HousingRequest.each do |hr|
      hr.set(main_applicant_id: nil)
    end 
    expect(Hermes::LogJob.queue_size).to eq(0)

    entity.update(siret: siret2, siren: siret2[0..8])
    execute_last_job(HousingRequestEntityUpdateJob)

    expect(Hermes::LogJob.queue_size).not_to eq(0)
  end 

end
